(* 
    File:  tilco.thy
    Theory Name: tilco
    Logic Image: Integ
*)

tilco = Interval +
types
  time    = int 
  tbool   = (time => bool)
  'a tfun = (time => 'a)
consts
  T        :: time
  now      :: [tbool, time] => bool

  true     :: tbool
  false    :: tbool

  "#~"     :: tbool => tbool                                   ("#~ _" [40] 40)
  "#&"     :: [tbool, tbool] => tbool                               (infixr 35)
  "#|"     :: [tbool, tbool] => tbool                               (infixr 30)
  "#="     :: ['a tfun, 'a tfun] => tbool                           (infixr 25)
  "#~="    :: [tbool, tbool] => tbool                               (infixr 33)
  "#-->"   :: [tbool, tbool] => tbool                               (infixr 25)
  TAll     :: ('a => tbool) => tbool                         (binder "#! "  10)
  TEx      :: ('a => tbool) => tbool                         (binder "#? "  10)
  TEx1     :: ('a => tbool) => tbool                         (binder "#?! " 10)
  TBall    :: ['a set, 'a => tbool] => tbool
  TBex     :: ['a set, 'a => tbool] => tbool

  at       :: [tbool, time set] => tbool                  ("_ @ _"  [36,37] 36)
  happen   :: [tbool, time set] => tbool                  ("_ ? _"  [36,37] 36)
  happen1  :: [tbool, time set] => tbool                  ("_ ?! _" [36,37] 36)
  "->>"    :: [tbool, tbool] => tbool                               (infixr 25)
  "-<<"    :: [tbool, tbool] => tbool                               (infixr 25)
  ":="     :: ['a tfun, 'a tfun] => tbool                           (infixr 38)
  until    :: [tbool, tbool] => tbool
  since    :: [tbool, tbool] => tbool

  up       :: tbool => tbool
  down     :: tbool => tbool
  tinv      :: ('a tfun) => tbool
  rule     :: tbool => bool
  fact     :: tbool => bool

  TIF      :: [tbool, 'a tfun, 'a tfun] => ('a tfun)
                                            ("(IF (_)/ THEN (_)/ ELSE (_))" 10)
  const    :: 'a => ('a tfun)
  unary    :: ['a => 'b, 'a tfun] => ('b tfun)
  binary   :: [['a,'b] => 'c,'a tfun,'b tfun] => ('c tfun)

syntax
  "*TBall" :: [idts,'a set,tbool]=>tbool  ("(3#! _:_./ _)" 10)
  "*TBex"  :: [idts,'a set,tbool]=>tbool  ("(3#? _:_./ _)"  10)
  "&&"     :: [time set, time set] => time set                      (infixr 40)
  "||"     :: [time set, time set] => time set                      (infixr 40)
  "@tEval":: 'a tfun => 'a tfun ("tEval[_]")
translations
  "A #~= B"          == "#~ (A #= B)"
  "#! x:A. P"        == "TBall A (%x. P)"
  "#? x:A. P"        == "TBex A (%x. P)"

  "A @   (i &&  j)"  => "(A @   i) #& (A @   j)"
  "A ?   (i &&  j)"  => "(A ?   i) #& (A ?   j)"
  "A ?!  (i &&  j)"  => "(A ?!  i) #& (A ?!  j)"
  "A @   (i ||  j)"  => "(A @   i) #| (A @   j)"
  "A ?   (i ||  j)"  => "(A ?   i) #| (A ?   j)"
  "A ?!  (i ||  j)"  => "(A ?!  i) #| (A ?!  j)"

defs
  (* T_def        "T            == #0" *)
  now_def      "now A t      == A(t + T)"

  Ttrue_def    "true         == (%x. True)"
  Tfalse_def   "false        == (%x. False)"

  Tnot_def     "#~ A         == (%x. ~ A x)"
  Tand_def     "A #& B       == (%x. A x & B x)"
  Tor_def      "A #| B       == (%x. A x | B x)"
  Tequiv_def   "A #= B       == (%x. A x = B x)"
  Timply_def   "A #--> B     == (%x. A x --> B x)"
  TAll_def     "TAll P       == (%y. ! x. P x y)"
  TEx_def      "TEx P        == (%y. ? x. P x y)"
  TEx1_def     "TEx1 P       == (%z. ?! x. P x z)"
  TBall_def    "TBall A P    == (%y. ! x:A. P x y)"
  TBex_def     "TBex A P     == (%y. ? x:A. P x y)"

  at_def       "A @  i       == (%x. ! t:i. A(t+x))"
  happen_def   "A ?  i       == (%x. ? t:i. A(t+x))" 
  happen1_def  "A ?! i       == (%x. ?! t. t:i & A(t+x))"
  Tnext_def    "A ->> B      == A #--> (B @ (single ($#1)))" 
  Tprev_def    "A -<< B      == A #--> (B @ (single ($~$#1)))"
  Tassign_def  "A := exp     == (%x. A x = exp (zpred x))"
  until_def    "until P Q    == (%x. ((Q @ <$#0,inf>) x) |
                                 (? t:<$#0,inf>.((Q @ <$#0,t>) x) & P(t+x)))"
  since_def    "since P Q    == (%x. ((Q @ <-inf,$#0>) x) |
                                 (? t:<-inf,$#0>.((Q @ <t,$#0>) x) & P(t+x)))"
  up_def       "up A         == (%t. ~A(zpred t) &  A t)"
  down_def     "down A       == (%t.  A(zpred t) & ~A t)"
  tinv_def     "tinv A        == (%x.  A x = (A (zpred x)) )"
  rule_def     "rule A       == (! t. now A t)"
  fact_def     "fact A       == (? t. now A t)"

  TIF_def      "TIF P x y    == (%z. if (P z) then (x z) else (y z))"
  const_def    "const K      == (%x. K)"
  unary_def    "unary f B    == (%t. f (B t))"
  binary_def   "binary f A B == (%t. f (A t) (B t))"

end






