(* 
    File:  ~/tilcoExt.thy
    Theory Name: tilcoExt
    Logic Image: tilco
*)

tilcoExt  =  tilco +

constdefs
  until0 :: [tbool,tbool] => tbool
    " until0 A B == A #| (B #& (until A B))"
  since0 :: [tbool,tbool] => tbool
    " since0 A B == A #| (B #& (since A B))"
  until' :: [tbool,tbool] => tbool
    " until' A B == A ? <$#0,inf> #& (until A B)"
  since' :: [tbool,tbool] => tbool
    " since' A B == A ? <-inf,$#0> #& (since A B)"
  until0' :: [tbool,tbool] => tbool
    " until0' A B == A ? [$#0,inf> #& (until0 A B)"
  since0' :: [tbool,tbool] => tbool
    " since0' A B == A ? <-inf,$#0] #& (since0 A B)"
  var :: 'a tfun => 'a tfun
    " var v == v"
end
ML

  fun tEval_tr [Const(c,t) $ term1 $ term2 ] =
       Const("binary",dummyT) $ Const(c,t) $ tEval_tr[term1] $ tEval_tr[term2]
    | tEval_tr [Free(v,t) $ term1 $ term2 ] =
       Const("binary",dummyT) $ Free(v,t) $ tEval_tr[term1] $ tEval_tr[term2]
    | tEval_tr [Const(c,t) $ term1] =
       Const("unary",dummyT) $ Const(c,t) $ tEval_tr[term1]
    | tEval_tr [Free("const",t) $ term1] =
       Free("const",t) $ term1 
    | tEval_tr [Free("var",t) $ term1] =
        Free("var",t) $term1 
    | tEval_tr [Free(v,t) $ term1] =
       Const("unary",dummyT) $ Free(v,t) $ tEval_tr[term1] 
    | tEval_tr [Const(c,t)] = Const("const",dummyT) $ Const(c,t)
    | tEval_tr [ a $ b] = 
       let
         exception exTEvalAbs;
         fun tEvalAbs_tr[a $ b] = tEvalAbs_tr[a] $ tEval_tr[b]
           | tEvalAbs_tr[Abs(v,t,b)] = Abs(v,t,tEval_tr[b])
           | tEvalAbs_tr ts = raise exTEvalAbs
       in
         tEvalAbs_tr[a $ b] handle exTEvalAbs => a $ b
       end
    | tEval_tr [Abs(v,t,b)] = Abs(v,t,tEval_tr[b])
    | tEval_tr [term] = term
    | tEval_tr ts = raise TERM ("tEval_tr", ts);
  val parse_translation = [("@tEval",tEval_tr)];

  local
    fun tEval_tr' [Const("binary",t) $ f $ a $ b] =
        f $ tEval_tr'[a] $ tEval_tr'[b]
      | tEval_tr' [Const("unary",t) $ f $ a]=
         f $ tEval_tr'[a]
      | tEval_tr' [Const("const",t) $ Const(c,t2) ]= Const(c,t2)
      | tEval_tr' [Const("var",t) $ Const(c,t2) ]= Const(c,t2)
      | tEval_tr' [t] = t
      | tEval_tr' ts = raise Match;
  in
    fun tEvalBin_tr' [Const(c,t) , a , b] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a] $ tEval_tr'[b])
      | tEvalBin_tr' [Const(c,t) , a , b , d] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a] $ tEval_tr'[b]) $d
      | tEvalBin_tr' [Free(v,t) , a , b] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a] $ tEval_tr'[b])
      | tEvalBin_tr' [Free(v,t) , a , b , c] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a] $ tEval_tr'[b]) $c
      | tEvalBin_tr' [t] = t
      | tEvalBin_tr' ts = raise Match;

    fun tEvalUna_tr' [Const(c,t) , a] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a])
      | tEvalUna_tr' [Const(c,t) , a, b] =
         Const("@tEval",dummyT) $ (Const(c,t) $ tEval_tr'[a]) $ b
      | tEvalUna_tr' [Free(v,t) , a] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a])
      | tEvalUna_tr' [Free(v,t) , a, b] =
         Const("@tEval",dummyT) $ (Free(v,t) $ tEval_tr'[a]) $ b
      | tEvalUna_tr' [t] = t
      | tEvalUna_tr' ts = raise Match;
  end
  val print_translation = [("binary",tEvalBin_tr'),("unary",tEvalUna_tr')];
